@@ -1,4 +1,4 @@
-/* $OpenBSD: misc.c,v 1.146 2020/01/28 01:49:36 djm Exp $ */
+/* $OpenBSD: misc.c,v 1.147 2020/04/25 06:59:36 dtucker Exp $ */
 /*
  * Copyright (c) 2000 Markus Friedl.  All rights reserved.
  * Copyright (c) 2005,2006 Damien Miller.  All rights reserved.
@@ -2255,11 +2255,13 @@ ssh_signal(int signum, sshsig_t handler)
 	struct sigaction sa, osa;
 
 	/* mask all other signals while in handler */
-	bzero(&sa, sizeof(sa));
+	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = handler;
 	sigfillset(&sa.sa_mask);
+#if defined(SA_RESTART) && !defined(NO_SA_RESTART)
 	if (signum != SIGALRM)
 		sa.sa_flags = SA_RESTART;
+#endif
 	if (sigaction(signum, &sa, &osa) == -1) {
 		debug3("sigaction(%s): %s", strsignal(signum), strerror(errno));
 		return SIG_ERR;
